import { FC, PropsWithChildren, useCallback, useMemo } from 'react';

import { Global, css } from '@emotion/react';
import createCache from '@emotion/cache';
import { ThemeProvider } from '@mui/material/styles';
import CssBaseline from '@mui/material/CssBaseline';

import { CacheProvider } from '@emotion/react';
import useMediaQuery from '@mui/material/useMediaQuery';

import nuskinTheme from '../themes/genome-foundation';
import { generateGlobalColorCss, createNuskinTheme, generateTextAlignCss } from '../utils';

import { FoundationTheme } from '../themes';

type NextGenThemeProviderProps = {
    isSSR?: boolean;
    cacheKey?: string;
    theme?: FoundationTheme.Theme;
    mode?: FoundationTheme.Mode;
};

const getCacheKey = (key: string) =>  createCache({ key });

const NextGenThemeProvider: FC<PropsWithChildren<NextGenThemeProviderProps>> = (props) => {
    
    const { isSSR = false, children, cacheKey = 'nextgen-theme-provider', theme = nuskinTheme, mode } = props;

    const prefersDarkMode = useMediaQuery('(prefers-color-scheme: dark)');

    const isDarkMode = mode ? mode === 'dark' : prefersDarkMode;
    
    const appMuiTheme = useMemo(() => {
        return createNuskinTheme(theme, isDarkMode);
    }, [isDarkMode, theme]);

    const cache = useMemo(() => getCacheKey(cacheKey), [cacheKey]);
    
    const renderThemeProvider = useCallback(
        () => (
            <ThemeProvider theme={appMuiTheme || {}}>
                <CssBaseline />
                <Global
                    styles={css`
                        ${generateTextAlignCss()}
                        ${generateGlobalColorCss(theme)}
                    `}
                />
                <main className={`theme-wrapper ${isDarkMode ? 'dark' : 'light'}`}>{children}</main>
            </ThemeProvider>
        ),
        [appMuiTheme, theme, isDarkMode, children]
    );

    if (isSSR) {
        return <CacheProvider value={cache}>{renderThemeProvider()}</CacheProvider>;
    }
    return renderThemeProvider();
};

export default NextGenThemeProvider;
