import { FoundationTheme } from 'src/themes';
import { ColorUtils } from '../color-utils';

const validTheme = {
    theme: {
        palette: {
            mode: 'light' as FoundationTheme.Mode,
            primary: {
                main: '#252525',
                contrastText: '#FFFFFF',
                buttonState: {
                    hovered: '#8C8C8C',
                    disabled: '#E0E0E0',
                    pressed: '#F5F5F5',
                    focused: '#252525',
                    disabledText: '#8C8C8C',
                },
                darkMode: {
                    main: '#F5F5F5',
                    contrastText: '#252525',
                    buttonState: {
                        hovered: '#C66464',
                        hoveredText: '#FFFFFF',
                        pressed: '#A54553',
                        pressedText: '#FFFFFF',
                        focused: '#F5F5F5',
                        disabled: '#5F5F5F',
                    },
                },
            },
        },
        colors: {
            primary: {
                dark: '#FFFFFF',
            },
            accentColorA: {
                20: '#F59F9F',
            },
        },
    },
};

describe('Color Utils', () => {
    it('1. getAppColor to check if it returns exact color value', () => {
        const color = ColorUtils.getAppColor(validTheme.theme, 'accentColorA', '20');
        expect(color).toBe('#F59F9F');
    });
    it('2. getAppColor to return empty string if it doesnt find variant', () => {
        const color = ColorUtils.getAppColor(validTheme.theme, 'primary', 'main');
        expect(color).toBe('');
    });
    it('3. color method to return hex value if it doesnt find variant', () => {
        const color = ColorUtils.color('primary', 'dark')(validTheme);
        expect(color).toBe('#FFFFFF');
    });
    it('4. color method to return empty string if it doesnt find variant', () => {
        const color = ColorUtils.color('accentColorD', 'main')(validTheme);
        expect(color).toBe('');
    });
    it('4. color method to return empty string if it doesnt find variant', () => {
        const color = ColorUtils.color('accentColorD', 'main')(validTheme);
        expect(color).toBe('');
    });
    it('5. check if dark mode false when mode === `light`', () => {
        const isDarkMode = ColorUtils.isDarkMode(validTheme);
        expect(isDarkMode).toBeFalsy();
    });
    it('6. check if dark mode true when mode === `dark`', () => {
        const darkModeTheme = {
            theme: {
                ...validTheme.theme,
                palette: {
                    ...validTheme?.theme?.palette,
                    mode: 'dark',
                }
            }
        };
        const isDarkMode = ColorUtils.isDarkMode(darkModeTheme);
        expect(isDarkMode).toBeTruthy();
    });
    it('7. check if dark mode true when mode === `dark`', () => {
        const darkModeTheme = {
            theme: {
                ...validTheme.theme,
                palette: {
                    ...validTheme?.theme?.palette,
                    mode: 'dark',
                },
            },
        };
        const isDarkMode = ColorUtils.isDarkMode(darkModeTheme);
        expect(isDarkMode).toBeTruthy();
    });
    it('8. getPaletteColor to check if it returns exact color value if variant is optional', () => {
        const color = ColorUtils.getPaletteColor(validTheme.theme, 'primary');
        expect(color).toBe('#252525');
    });
    it('9. getPaletteColor to return palette colors contrastText value', () => {
        const color = ColorUtils.getPaletteColor(validTheme.theme, 'primary', 'contrastText');
        expect(color).toBe('#FFFFFF');
    });
    it('10. paletteColor to return palette colors value', () => {
        const color = ColorUtils.paletteColor('primary')(validTheme);
        expect(color).toBe('#252525');
    });
    it('11. paletteColor to return empty string if palette color is not found', () => {
        const color = ColorUtils.paletteColor('notFoundPalette' as FoundationTheme.PaletteColors)(validTheme);
        expect(color).toBe('');
    });
    it('12. buttonColor to return the button state color', () => {
        const color = ColorUtils.buttonColor('primary', 'hovered')(validTheme);
        expect(color).toBe('#8C8C8C');
    });
    it('13. buttonColor to empty string if state is not found', () => {
        const color = ColorUtils.buttonColor('primary', 'customState' as keyof FoundationTheme.ButtonState)(validTheme);
        expect(color).toBe('');
    });
    it('14. getButtonState to return the button state color', () => {
        const color = ColorUtils.getButtonStateColor(validTheme.theme, 'primary', 'hovered');
        expect(color).toBe('#8C8C8C');
    });
    it('15. paletteColor to return dark color value is darkModeEnabled is enabled', () => {
        const darkModeTheme = {
            theme: {
                ...validTheme.theme,
                palette: {
                    ...validTheme?.theme?.palette,
                    success: {
                        main: '#light',
                        darkMode: {
                            main: '#dark'
                        }
                    },
                    mode: 'dark',
                },
            },
        };
        const color = ColorUtils.paletteColor('success')(darkModeTheme);
        expect(color).toBe('#dark');
    });
    it('16. paletteColor to return light mode value is darkModeEnabled is off', () => {
        const darkModeTheme = {
            darkModeEnabled: false,
            theme: {
                ...validTheme.theme,
                palette: {
                    ...validTheme?.theme?.palette,
                    success: {
                        main: '#light',
                        darkMode: {
                            main: '#dark',
                        },
                    },
                    mode: 'dark',
                },
            },
        };
        const color = ColorUtils.paletteColor('success')(darkModeTheme);
        expect(color).toBe('#light');
    });
});
