import { generateGlobalColorCss, generatePalette, generateTextAlignCss, createNuskinTheme } from '../theme-utils';

const validTheme = {
    'palette': {
        'primary': {
            'main': '#252525',
            'contrastText': '#FFFFFF',
            'buttonState': {
                'hovered': '#8C8C8C',
                'disabled': '#E0E0E0',
                'pressed': '#F5F5F5',
                'focused': '#252525',
                'disabledText': '#8C8C8C'
            },
            'darkMode': {
                'main': '#F5F5F5',
                'contrastText': '#252525',
                'buttonState': {
                    'hovered': '#C66464',
                    'hoveredText': '#FFFFFF',
                    'pressed': '#A54553',
                    'pressedText': '#FFFFFF',
                    'focused': '#F5F5F5',
                    'disabled': '#5F5F5F'
                }
            }
        },
    },
};

describe('Theme Utils', () => {
    it('1. generatePalette to return light mode values', () => {
        const palette = generatePalette(validTheme.palette, false);
        expect(palette?.primary?.main).toBe('#252525');
    });
    
    it('2. generatePalette to return dark mode values', () => {
        const palette = generatePalette(validTheme.palette, true);
        expect(palette?.primary?.main).toBe('#F5F5F5');
    });

    it('3. generateGlobalColorCss to return the proper css string', () => {
        const cssString = generateGlobalColorCss(validTheme);
        expect(cssString).toContain('.bg-primary');
        expect(cssString).toContain('.text-primary');
        expect(cssString).toContain('.border-primary');
    });

    it('4. generateGlobalColorCss to return the proper css string', () => {
        const cssString = generateGlobalColorCss(validTheme);
        expect(cssString).toContain('.bg-primary');
        expect(cssString).toContain('.text-primary');
        expect(cssString).toContain('.border-primary');
    });

    it('5. generateGlobalColorCss to have the .dark class', () => {
        const theme = {
            ...validTheme,
            colors: {
                'accentColorA': {
                    main: '#F56767',
                    dark: '#FC9492'
                }
            },
        };
        const cssString = generateGlobalColorCss(theme);
        expect(cssString).toContain('.dark {');
    });

    it('6. generateGlobalColorCss to not have the .dark class', () => {
        const cssString = generateGlobalColorCss(validTheme);
        expect(cssString).not.toContain('.dark {');
    });

    it('7. generateGlobalColorCss to return empty string if theme is falsy', () => {
        const cssString = generateGlobalColorCss(null as any);
        expect(cssString).toBe('');
    });

    it('8. generateTextAlignCss to output all text align classes', () => {
        const cssString = generateTextAlignCss();
        expect(cssString).toContain('.text-left');
        expect(cssString).toContain('.text-right');
        expect(cssString).toContain('.text-center');
        expect(cssString).toContain('.text-justify');
        expect(cssString).toContain('.text-start');
        expect(cssString).toContain('.text-end');
    });

    it('9. createNuskinTheme to inherit from MUI Theme', () => {
        const theme = createNuskinTheme(validTheme);
        expect(theme?.palette?.primary?.main).toBe('#252525');
    });
});