import applySpec from 'ramda/src/applySpec';
import compose from 'ramda/src/compose';
import pathOr from 'ramda/src/pathOr';
import propEq from 'ramda/src/propEq';

import { FoundationTheme } from '..';

import { isValidPaletteColor, isValidAppColorGroup } from './common-utils';

type PaletteColor = null | {
    main?: string;
    contrastText?: string;
    darkMode?: { main: string; contrastText: string };
};

const getThemeProps = applySpec<{
    theme: FoundationTheme.Theme;
    darkModeEnabled?: boolean;
}>({
    theme: pathOr({}, ['theme']),
    darkModeEnabled: pathOr(true, ['darkModeEnabled']),
});

const getTheme = pathOr({}, ['theme']);

/** @deprecated - use `ColorUtils.color` */
export const getAppColor = (
    theme: FoundationTheme.Theme,
    colorGroup: keyof FoundationTheme.Colors,
    variant?: string
) => {
    const { colors } = theme || {};
    if (colorGroup && variant) {
        const appColor = isValidAppColorGroup(colorGroup) ? colorGroup : 'primary';
        return colors?.[appColor]?.[variant] || '';
    }
    return '';
};

/**
   * Returns the color hex value from the theme's color object.
   *
   * @example
   * theme = {
   *    colors: {
   *        [colorType]: {
   *            [variant]: HexValue
   *        }
   *    }
   * }
   *
   * @param colorName - The color type has to be passed
   * Examples: primary | neutral | accentColorA | accentColorB | accentColorC | 
   * accentColorD | success | warning | red | info | custom 
   * @param variant - Each color type has different variant
   * Examples: 10 | 20 | 30 | 40 | 100 | 200 | 300
   * @returns The hex value if color and variant are found.
   *
   */
const color = (colorType: keyof FoundationTheme.Colors, variant: string | number) =>
    compose(pathOr('', ['colors', colorType, variant]), getTheme);

/**
   * isDarkMode verifies if the current theme has dark theme enabled.
   * {
   *    theme: {
   *        mode === "dark"
   *    }
   * }
   * 
   * @returns true if current theme's mode is dark.
   *
   */
const isDarkMode = compose(propEq('dark', 'mode'), pathOr({ mode: null }, ['palette']), getTheme);

/** @deprecated - use `ColorUtils.paletteColor` */
export const getPaletteColor = (
    theme: FoundationTheme.Theme,
    color: FoundationTheme.PaletteColors = 'primary',
    variant?: 'main' | 'contrastText'
) => {
    const { palette } = theme;
    const paletteColor = isValidPaletteColor(color) ? color : 'primary';
    return palette?.[paletteColor]?.[variant || 'main'] || '';
};

const getPalette = (color: FoundationTheme.PaletteColors = 'primary') => compose(
    function (appTheme) {
        const { theme, darkModeEnabled } = appTheme;
        const currentPalette: PaletteColor = pathOr({}, ['palette', color])(theme);
        const darkMode = currentPalette?.darkMode;
        if (isDarkMode(appTheme) && darkModeEnabled && darkMode?.main) {
            return {
                ...currentPalette,
                ...darkMode,
            };
        }
        return currentPalette;
    },
    getThemeProps,
);

/**
   * Returns the color hex value from the theme's color object.
   *
   * Note: This will not switch between dark and light mode if the darkModeEnabled is passed as false, 
   * from the component.
   * Also if there is no darkMode property avaiable in theme's palette for the particular color it will not switch mode.
   * 
   * @example
   * theme = {
   *    palette: {
   *        [paletteColorType]: {
   *            main: HexColorString,
   *            contrastText: HexColorString,
   *            darkMode: {
   *                main: HexColorString,
   *                contrastText: HexColorString,
   *            },
   *            buttonState: {
   *                hovered: HexColorString,
   *                pressed: HexColorString,
   *                focused: HexColorString,
   *                disabled: HexColorString,
   *                disabledText: HexColorString
   *            }
   *        }
   *    }
   * }
   *
   * @param paletteColorType - The palette color that has to be passed
   * Examples: primary | neutral | accentColorA | accentColorB | accentColorC | 
   * accentColorD | success | warning | red | info 
   * @param variant - Each color type has different variant
   * Examples: main | contrastText
   * @returns The hex value if color and variant are found.
   *
   */

const paletteColor = (paletteColorType: FoundationTheme.PaletteColors = 'primary', variant?: 'main' | 'contrastText') =>
    compose(pathOr('', [variant || 'main']), getPalette(paletteColorType));

/** @deprecated - use `ColorUtils.buttonColor` */
export const getButtonStateColor = (
    theme: FoundationTheme.Theme,
    color: FoundationTheme.PaletteColors = 'primary',
    state: keyof FoundationTheme.ButtonState
) => {
    const { palette } = theme || {};
    const paletteColor = isValidPaletteColor(color) ? color : 'primary';
    return palette?.[paletteColor]?.buttonState?.[state] || '';
};

/**
   * Returns the button color hex value according to the state .
   *
   * Note: This will not switch between dark and light mode if the darkModeEnabled is passed as false, 
   * from the component.
   * 
   * @example
   * theme = {
   *    palette: {
   *        [paletteColorType]: {
   *            main: HexColorString,
   *            contrastText: HexColorString,
   *            darkMode: {
   *                main: HexColorString,
   *                contrastText: HexColorString,
   *            },
   *            buttonState: {
   *                hovered: HexColorString,
   *                pressed: HexColorString,
   *                focused: HexColorString,
   *                disabled: HexColorString,
   *                disabledText: HexColorString
   *            }
   *        }
   *    }
   * }
   *
   * @param paletteColorType - The palette color that has to be passed
   * Examples: primary | neutral | accentColorA | accentColorB | accentColorC | 
   * accentColorD | success | warning | red | info 
   * @param state - Each color type has different variant
   * Examples: hovered | pressed | focused | disabled | disabledText
   * @returns The hex value if color and variant are found.
   *
   */

const getButtonState = (state: string) => pathOr('', ['buttonState', state]);

const buttonColor = (
    paletteColorType: FoundationTheme.PaletteColors = 'primary',
    state: keyof FoundationTheme.ButtonState
) => compose(
    getButtonState(state),
    getPalette(paletteColorType)
);


export const ColorUtils = {
    getButtonStateColor,
    getPaletteColor,
    getAppColor,
    color,
    paletteColor,
    buttonColor,
    isDarkMode,
};
